@@ -7,6 +7,7 @@ Copyright 2013 John Evans
 License:  MIT
 """
 # Standard library imports...
+from __future__ import annotations
 from collections import Counter
 from contextlib import ExitStack
 from itertools import filterfalse
@@ -15,6 +16,7 @@ import pathlib
 import re
 import shutil
 import struct
+from typing import List, Tuple
 from uuid import UUID
 import warnings
 
@@ -34,19 +36,100 @@ from .lib import openjp2 as opj2
 
 
 class Jp2k(Jp2kBox):
-    """JPEG 2000 file.
+    """Access JPEG 2000 files.
+
+    Parameters
+    ----------
+    filename : str or path
+        The path to JPEG 2000 file.  If you are only reading JPEG 2000 files,
+        this is the only argument you need to supply.
+    data : np.ndarray, optional
+        Image data to be written to file.
+    shape : Tuple[int, int, ...], optional
+        Size of image data, only required when image_data is not provided.
+    capture_resolution : Tuple[int, int], optional
+        Capture solution (VRES, HRES).  This appends a capture resolution
+        box onto the end of the JP2 file when it is created.
+    cbsize : Tuple[int, int], optional
+        Code block size (NROWS, NCOLS)
+    cinema2k : int, optional
+        Frames per second, either 24 or 48.
+    cinema4k : bool, optional
+        Set to True to specify Cinema4K mode, defaults to false.
+    colorspace : {'rgb', 'gray'}, optional
+        The image color space.  If not supplied, it will be inferred.
+    cratios : Tuple[int, ...], optional
+        Compression ratios for successive layers.
+    display_resolution : Tuple[int, int], optional
+        Display solution (VRES, HRES).  This appends a display resolution
+        box onto the end of the JP2 file when it is created.
+    eph : bool, optional
+        If true, write EPH marker after each header packet.
+    grid_offset : Tuple[int, int], optional
+        Offset (DY, DX) of the origin of the image in the reference grid.
+    irreversible : bool, optional
+        If true, use the irreversible DWT 9-7 transform.
+    mct : bool, optional
+        Usage of the multi component transform to write an image.  If not
+        specified, defaults to True if the color space is RGB, false if the
+        color space is grayscale.
+    modesw : int, optional
+        mode switch
+            1 = BYPASS(LAZY)
+            2 = RESET
+            4 = RESTART(TERMALL)
+            8 = VSC
+            16 = ERTERM(SEGTERM)
+            32 = SEGMARK(SEGSYM)
+    numres : int, optional
+        Number of resolutions, defaults to 6.  This number will be equal to
+        the number of thumbnails plus the original image.
+    plt : bool, optional
+        Generate PLT markers.
+    prog : {'LRCP', 'RLCP', 'RPCL', 'PCRL', 'CPRL'}, optional
+        Progression order.  If not specified, the chosen progression order
+        will be 'CPRL' if either cinema2k or cinema4k is specified,
+        otherwise defaulting to 'LRCP'.
+    psizes : List[Tuple[int, int]], optional
+        Precinct sizes, each precinct size tuple is defined as
+        (height, width).
+    psnr : Tuple[int, ...] or None
+        Different PSNR for successive layers.  If the last layer is desired
+        to be lossless, specify 0 for the last value.
+    sop : bool, optional
+        If true, write SOP marker before each packet.
+    subsam : Tuple[int, int], optional
+        Subsampling factors (dy, dx).
+    tilesize : Tuple[int, int], optional
+        Tile size in terms of (numrows, numcols), not (X, Y).
+    tlm : bool, optional
+        Generate TLM markers.
+    verbose : bool, optional
+        Print informational messages produced by the OpenJPEG library.
 
     Attributes
     ----------
-    codestream
-    decoded_components
-    dtype
-    ignore_pclr_cmap_cdef
-    layer
-    ndim
-    shape
-    tilesize
-    verbose
+    codestream : glymur.codestream.Codestream
+        Contains header information for the codestream.
+    decoded_components : bool
+        If set, decode only these components.  The MCT will not be used.
+    dtype : numpy dtype object
+        Describes the format of the elements in the image data.
+    ignore_pclr_cmap_cdef : bool
+        Specifies whether or not to ignore the pclr, cmap, or cdef
+        boxes during any color transformation, defaults to False.
+    layer : int
+        Zero-based number of quality layer to decode.  Defaults to 0, the
+        highest quality layer.
+    ndim : ndim
+        The image data array's number of dimensions.
+    shape : tuple of ints
+        Shape of the image data array.
+    tilesize : 2-tuple of ints
+        Height and width of the tiles that make up the image.
+    verbose : bool
+        Specifies whether or not to print informational messages
+        produced by the OpenJPEG library.
     filename : str
         The path to the JPEG 2000 file.
     box : sequence
@@ -90,82 +173,33 @@ class Jp2k(Jp2kBox):
     """
 
     def __init__(
-        self, filename, data=None, shape=None, tilesize=None, verbose=False,
-        capture_resolution=None, cbsize=None, cinema2k=None,
-        cinema4k=None, colorspace=None, cratios=None,
-        display_resolution=None, eph=None, grid_offset=None,
-        irreversible=None, mct=None, modesw=None, numres=None,
-        plt=False, prog=None, psizes=None, psnr=None, sop=None,
-        subsam=None, tlm=False,
+        self,
+        filename: str | pathlib.Path,
+        data: np.ndarray | None = None,
+        capture_resolution: Tuple[int, int] | None = None,
+        cbsize: Tuple[int, int] | None = None,
+        cinema2k: int = 0,
+        cinema4k: bool = False,
+        colorspace: str | None = None,
+        cratios: Tuple[int, ...] | None = None,
+        display_resolution: Tuple[int, int] | None = None,
+        eph: bool = False,
+        grid_offset: Tuple[int, int] | None = None,
+        irreversible: bool = False,
+        mct: bool | None = None,
+        modesw: int = 0,
+        numres: int = 6,
+        plt: bool = False,
+        prog: str | None = None,
+        psizes: List[Tuple[int, int]] | None = None,
+        psnr: Tuple[int, ...] | None = None,
+        shape: Tuple[int, int, ...] | None = None,
+        sop: bool = False,
+        subsam: Tuple[int, int] | None = None,
+        tilesize: Tuple[int, int] | None = None,
+        tlm: bool = False,
+        verbose: bool = False,
     ):
-        """Construct a Jp2k object.
-
-        Parameters
-        ----------
-        filename : str
-            The path to JPEG 2000 file.
-        path : pathlib.Path
-            The path to JPEG 2000 file.
-        image_data : ndarray, optional
-            Image data to be written to file.
-        shape : tuple, optional
-            Size of image data, only required when image_data is not provided.
-        capture_resolution : tuple, optional
-            Capture solution (VRES, HRES).  This appends a capture resolution
-            box onto the end of the JP2 file when it is created.
-        cbsize : tuple, optional
-            Code block size (NROWS, NCOLS)
-        cinema2k : int, optional
-            Frames per second, either 24 or 48.
-        cinema4k : bool, optional
-            Set to True to specify Cinema4K mode, defaults to false.
-        colorspace : {'rgb', 'gray'}
-            The image color space.
-        cratios : iterable, optional
-            Compression ratios for successive layers.
-        display_resolution : tuple, optional
-            Display solution (VRES, HRES).  This appends a display resolution
-            box onto the end of the JP2 file when it is created.
-        eph : bool, optional
-            If true, write EPH marker after each header packet.
-        grid_offset : tuple, optional
-            Offset (DY, DX) of the origin of the image in the reference grid.
-        irreversible : bool, optional
-            If true, use the irreversible DWT 9-7 transform.
-        mct : bool, optional
-            Usage of the multi component transform to write an image.  If not
-            specified, defaults to True if the color space is RGB, false if the
-            color space is grayscale.
-        modesw : int, optional
-            mode switch
-                1 = BYPASS(LAZY)
-                2 = RESET
-                4 = RESTART(TERMALL)
-                8 = VSC
-                16 = ERTERM(SEGTERM)
-                32 = SEGMARK(SEGSYM)
-        numres : int, optional
-            Number of resolutions.
-        plt : bool, optional
-            Generate PLT markers.
-        prog : {"LRCP" "RLCP", "RPCL", "PCRL", "CPRL"}
-            Progression order.
-        psnr : iterable, optional
-            Different PSNR for successive layers.
-        psizes : list, optional
-            List of precinct sizes, each precinct size tuple is defined in
-            (height x width).
-        sop : bool, optional
-            If true, write SOP marker before each packet.
-        subsam : tuple, optional
-            Subsampling factors (dy, dx).
-        tilesize : tuple, optional
-            Tile size in terms of (numrows, numcols), not (X, Y).
-        tlm : bool, optional
-            Generate TLM markers.
-        verbose : bool, optional
-            Print informational messages produced by the OpenJPEG library.
-        """
         super().__init__()
 
         # In case of pathlib.Paths...
@@ -189,7 +223,10 @@ class Jp2k(Jp2kBox):
         self._irreversible = irreversible
         self._mct = mct
         self._modesw = modesw
-        self._numres = numres
+
+        # The default used to be 6 if numres came in as None.
+        self._numres = numres if numres is not None else 6
+
         self._plt = plt
         self._prog = prog
         self._psizes = psizes
@@ -317,13 +354,11 @@ class Jp2k(Jp2kBox):
         non_cinema_args = (
             self._mct, self._cratios, self._psnr, self._irreversible,
             self._cbsize, self._eph, self._grid_offset, self._modesw,
-            self._numres, self._prog, self._psizes, self._sop, self._subsam
+            self._prog, self._psizes, self._sop, self._subsam
         )
         if (
-            (
-                self._cinema2k is not None or self._cinema4k is not None
-            )
-            and (not all([arg is None for arg in non_cinema_args]))
+            (self._cinema2k or self._cinema4k)
+            and not all([arg is None or not arg for arg in non_cinema_args])
         ):
             msg = (
                 "Cannot specify cinema2k/cinema4k along with any other "
@@ -337,19 +372,15 @@ class Jp2k(Jp2kBox):
                 raise InvalidJp2kError(msg)
 
             if 0 in self._psnr and self._psnr[-1] != 0:
-                msg = ("If a zero value is supplied in the PSNR keyword "
-                       "argument, it must be in the final position.")
+                msg = (
+                    "If a zero value is supplied in the PSNR keyword "
+                    "argument, it must be in the final position."
+                )
                 raise InvalidJp2kError(msg)
 
             if (
-                (
-                    0 in self._psnr
-                    and np.any(np.diff(self._psnr[:-1]) < 0)
-                )
-                or (
-                    0 not in self._psnr
-                    and np.any(np.diff(self._psnr) < 0)
-                )
+                0 in self._psnr and np.any(np.diff(self._psnr[:-1]) < 0)
+                or 0 not in self._psnr and np.any(np.diff(self._psnr) < 0)
             ):
                 msg = (
                     "PSNR values must be increasing, with one exception - "
@@ -358,10 +389,7 @@ class Jp2k(Jp2kBox):
                 )
                 raise InvalidJp2kError(msg)
 
-        if (
-            self._codec_format == opj2.CODEC_J2K
-            and self._colorspace is not None
-        ):
+        if self._codec_format == opj2.CODEC_J2K and self._colorspace is not None:  # noqa : E501
             msg = 'Do not specify a colorspace when writing a raw codestream.'
             raise InvalidJp2kError(msg)
 
@@ -676,8 +704,9 @@ class Jp2k(Jp2kBox):
         # A jp2-branded file cannot contain an "any ICC profile
         colrs = [box for box in jp2h.box if box.box_id == 'colr']
         for colr in colrs:
-            if colr.method not in (core.ENUMERATED_COLORSPACE,
-                                   core.RESTRICTED_ICC_PROFILE):
+            if colr.method not in (
+                core.ENUMERATED_COLORSPACE, core.RESTRICTED_ICC_PROFILE
+            ):
                 msg = (
                     "Color Specification box method must specify either an "
                     "enumerated colorspace or a restricted ICC profile if the "
@@ -782,11 +811,12 @@ class Jp2k(Jp2kBox):
 
         cparams.irreversible = 1 if self._irreversible else 0
 
-        if self._cinema2k is not None:
+        if self._cinema2k:
+            # cinema2k is an integer, so this test is "truthy"
             self._cparams = cparams
             self._set_cinema_params('cinema2k', self._cinema2k)
 
-        if self._cinema4k is not None:
+        if self._cinema4k:
             self._cparams = cparams
             self._set_cinema_params('cinema4k', self._cinema4k)
 
@@ -808,13 +838,13 @@ class Jp2k(Jp2kBox):
             cparams.image_offset_y0 = self._grid_offset[0]
 
         if self._modesw is not None:
+            # The None check is for backwards compatibility.
             for shift in range(6):
                 power_of_two = 1 << shift
                 if self._modesw & power_of_two:
                     cparams.mode |= power_of_two
 
-        if self._numres is not None:
-            cparams.numresolution = self._numres
+        cparams.numresolution = self._numres
 
         if self._prog is not None:
             cparams.prog_order = core.PROGRESSION_ORDER[self._prog.upper()]
@@ -851,7 +881,6 @@ class Jp2k(Jp2kBox):
 
             # the MCT was requested, but the colorspace is gray
             # i.e. 1 component.  NOT ON MY WATCH!
-
             msg = (
                 "You cannot specify usage of the multi component transform "
                 "if the colorspace is gray."
@@ -860,7 +889,7 @@ class Jp2k(Jp2kBox):
 
         else:
 
-            # The MCT was explicitly not specified
+            # The MCT was either not specified
             # or it was specified AND the colorspace is going to be RGB.
             # In either case, we can use the MCT as requested.
             cparams.tcp_mct = 1 if self._mct else 0
@@ -1084,26 +1113,32 @@ class Jp2k(Jp2kBox):
             opj2.end_compress(codec, strm)
 
     def append(self, box):
-        """Append a JP2 box to the file in-place.
+        """Append a metadata box to the JP2 file.  This will not result in a
+        file-copy operation.  Only XML UUID (XMP), or ASOC boxes can be
+        appended at this time.
 
         Parameters
         ----------
         box : Jp2Box
-            Instance of a JP2 box.  Only UUID and XML boxes can currently be
-            appended.
+            Instance of a JP2 box.
         """
         if self._codec_format == opj2.CODEC_J2K:
-            msg = "Only JP2 files can currently have boxes appended to them."
+            msg = "You cannot append to a J2K file (raw codestream)."
             raise RuntimeError(msg)
 
+        box_is_asoc = box.box_id == 'asoc'
         box_is_xml = box.box_id == 'xml '
         box_is_xmp = (
             box.box_id == 'uuid'
-            and box.uuid == UUID('be7acfcb-97a9-42e8-9c71-999491e3afac')
+            and (
+                box.uuid == UUID('be7acfcb-97a9-42e8-9c71-999491e3afac')
+                or box.uuid == UUID('b14bf8bd-083d-4b43-a5ae-8cd7d5a6ce03')
+            )
         )
-        if not (box_is_xml or box_is_xmp):
+        if not (box_is_asoc or box_is_xml or box_is_xmp):
             msg = (
-                "Only XML boxes and XMP UUID boxes can currently be appended."
+                "Only ASOC, XML, or UUID (XMP or GeoTIFF) boxes can currently "
+                "be appended."
             )
             raise RuntimeError(msg)
 
@@ -1426,30 +1461,6 @@ class Jp2k(Jp2kBox):
         # Ok, 3 arguments in pargs.
         return data[:, :, bands]
 
-    def _read(self, **kwargs):
-        """Read a JPEG 2000 image.
-
-        Returns
-        -------
-        ndarray
-            The image data.
-
-        Raises
-        ------
-        RuntimeError
-            if the proper version of the OpenJPEG library is not available
-        """
-        if re.match("0|1|2.[012]", version.openjpeg_version):
-            msg = (
-                f"You must have a version of OpenJPEG at least as high as "
-                f"2.3.0 before you can read JPEG2000 images with glymur.  "
-                f"Your version is {version.openjpeg_version}"
-            )
-            raise RuntimeError(msg)
-
-        img = self._read_openjp2(**kwargs)
-        return img
-
     def read(self, **kwargs):
         """Read a JPEG 2000 image.
 
@@ -1490,8 +1501,7 @@ class Jp2k(Jp2kBox):
             )
             raise RuntimeError(msg)
 
-    def _read_openjp2(self, rlevel=0, layer=None, area=None, tile=None,
-                      verbose=False):
+    def _read(self, rlevel=0, layer=None, area=None, tile=None, verbose=False):
         """Read a JPEG 2000 image using libopenjp2.
 
         Parameters
@@ -1519,12 +1529,20 @@ class Jp2k(Jp2kBox):
         RuntimeError
             If the image has differing subsample factors.
         """
+        if re.match("0|1|2.[012]", version.openjpeg_version):
+            msg = (
+                f"You must have a version of OpenJPEG at least as high as "
+                f"2.3.0 before you can read JPEG2000 images with glymur.  "
+                f"Your version is {version.openjpeg_version}"
+            )
+            raise RuntimeError(msg)
+
         self._subsampling_sanity_check()
         self._populate_dparams(rlevel, tile=tile, area=area)
-        image = self._read_openjp2_common()
+        image = self._read_openjp2()
         return image
 
-    def _read_openjp2_common(self):
+    def _read_openjp2(self):
         """Read a JPEG 2000 image using libopenjp2.
 
         Returns
@@ -1692,7 +1710,7 @@ class Jp2k(Jp2kBox):
         self.ignore_pclr_cmap_cdef = ignore_pclr_cmap_cdef
         self.layer = layer
         self._populate_dparams(rlevel, tile=tile, area=area)
-        lst = self._read_openjp2_common()
+        lst = self._read_openjp2()
         return lst
 
     def _extract_image(self, raw_image):
@@ -1738,15 +1756,18 @@ class Jp2k(Jp2kBox):
             addr = ctypes.addressof(component.data.contents)
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore")
-                nelts = nrows[k] * ncols[k]
-                band = np.ctypeslib.as_array(
-                    (ctypes.c_int32 * nelts).from_address(addr))
+
+                band_i32 = np.ctypeslib.as_array(
+                    (ctypes.c_int32 * nrows[k] * ncols[k]).from_address(addr)
+                )
+                band = np.reshape(
+                    band_i32.astype(dtypes[k]), (nrows[k], ncols[k])
+                )
+
                 if is_cube:
-                    image[:, :, k] = np.reshape(band.astype(dtypes[k]),
-                                                (nrows[k], ncols[k]))
+                    image[:, :, k] = band
                 else:
-                    image.append(np.reshape(band.astype(dtypes[k]),
-                                 (nrows[k], ncols[k])))
+                    image.append(band)
 
         if is_cube and image.shape[2] == 1:
             # The third dimension has just a single layer.  Make the image
