@@ -2,12 +2,12 @@
 import importlib.resources as ir
 import logging
 import pathlib
-import platform
 import shutil
 import struct
 import sys
 import tempfile
 import unittest
+from unittest.mock import patch
 from uuid import UUID
 import warnings
 
@@ -42,6 +42,7 @@ class TestSuite(fixtures.TestCommon):
 
         cls.astronaut8 = _file_helper('astronaut8.tif')
         cls.astronaut_u16 = _file_helper('astronaut_uint16.tif')
+        cls.astronaut_s_u16 = _file_helper('astronaut_s_uint16.tif')
         cls.astronaut8_stripped = _file_helper('astronaut8_stripped.tif')
         cls.astronaut_ycbcr_jpeg_tiled = _file_helper('astronaut_ycbcr_jpeg_tiled.tif')  # noqa : E501
         cls.moon = _file_helper('moon.tif')
@@ -49,6 +50,7 @@ class TestSuite(fixtures.TestCommon):
         cls.moon_3stripped = _file_helper('moon3_stripped.tif')
         cls.moon3_partial_last_strip = _file_helper('moon3_partial_last_strip.tif')  # noqa : E501
         cls.ycbcr_bg = _file_helper('ycbcr_bg.tif')
+        cls.ycbcr_stripped = _file_helper('ycbcr_stripped.tif')
         cls.stripped = _file_helper('stripped.tif')
 
         test_tiff_dir = tempfile.mkdtemp()
@@ -137,10 +139,9 @@ class TestSuite(fixtures.TestCommon):
             f.write(buffer)
 
             # XMP
-            xmp_path = fixtures._path_to('issue555.xmp')
-            with xmp_path.open() as f2:
-                xmp = f2.read()
-                xmp = xmp + '\0'
+            xmp_path = ir.files('tests.data').joinpath('issue555.xmp')
+            txt = xmp_path.read_text()
+            xmp = txt + '\0'
             buffer = struct.pack(
                 '<HHII', 700, 1, len(xmp), main_ifd_data_offset + 32
             )
@@ -260,9 +261,9 @@ class TestSuite(fixtures.TestCommon):
         self.assertEqual(j.box[-1].data['ImageWidth'], 512)
         self.assertEqual(j.box[-1].data['ImageLength'], 512)
 
-    def test_smoke_rgba(self):
+    def test_smoke_jp2_tiling(self):
         """
-        SCENARIO:  Convert RGBA TIFF file to JP2
+        SCENARIO:  Convert RGB TIFF file to a tiled JP2.
 
         EXPECTED RESULT:  data matches, number of resolution is the default.
         There should be just one layer.  The number of resolutions should be
@@ -344,14 +345,11 @@ class TestSuite(fixtures.TestCommon):
         )
         self.assertFalse(at_least_one_uuid)
 
-    @unittest.skipIf(
-        platform.machine() == 's390x', 'See issue #546'
-    )
     def test_psnr(self):
         """
-        SCENARIO:  Convert TIFF file to JP2 with the irreversible transform.
+        SCENARIO:  Convert TIFF file to JP2 with the psnr keyword argument
 
-        EXPECTED RESULT:  data matches, the irreversible transform is confirmed
+        EXPECTED RESULT:  data matches
         """
         with Tiff2Jp2k(
             self.moon, self.temp_jp2_filename, psnr=(30, 35, 40, 0)
@@ -385,6 +383,45 @@ class TestSuite(fixtures.TestCommon):
         # PSNR should increase for the remaining images.
         self.assertTrue(np.all(np.diff(psnr[1:])) > 0)
 
+    def test_psnr_commandline(self):
+        """
+        SCENARIO:  Convert TIFF file to JP2, specify psnr via the command line
+
+        EXPECTED RESULT:  data matches
+        """
+        sys.argv = [
+            '', str(self.moon), str(self.temp_jp2_filename),
+            '--psnr', '30', '35', '40', '0'
+        ]
+        command_line.tiff2jp2()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        d = {}
+        for layer in range(4):
+            j.layer = layer
+            d[layer] = j[:]
+
+        truth = skimage.io.imread(self.moon)
+
+        with warnings.catch_warnings():
+            # MSE is zero for that first image, resulting in a divide-by-zero
+            # warning
+            warnings.simplefilter('ignore')
+            psnr = [
+                skimage.metrics.peak_signal_noise_ratio(truth, d[j])
+                for j in range(4)
+            ]
+
+        # That first image should be lossless.
+        self.assertTrue(np.isinf(psnr[0]))
+
+        # None of the subsequent images should have inf PSNR.
+        self.assertTrue(not np.any(np.isinf(psnr[1:])))
+
+        # PSNR should increase for the remaining images.
+        self.assertTrue(np.all(np.diff(psnr[1:])) > 0)
+
     def test_irreversible(self):
         """
         SCENARIO:  Convert TIFF file to JP2 with the irreversible transform.
@@ -529,6 +566,28 @@ class TestSuite(fixtures.TestCommon):
         c = j.get_codestream()
         self.assertEqual(c.segment[2].layers, 3)
 
+    def test_layers_commandline(self):
+        """
+        SCENARIO:  Convert TIFF file to JP2 with multiple compression layers
+        using the command line.
+
+        EXPECTED RESULT:  data matches, number of layers is 3
+        """
+        sys.argv = [
+            '',
+            str(self.astronaut_ycbcr_jpeg_tiled), str(self.temp_jp2_filename),
+            '--cratio', '200', '50', '10'
+        ]
+        command_line.tiff2jp2()
+
+        j = Jp2k(self.temp_jp2_filename)
+
+        actual = j[:]
+        self.assertEqual(actual.shape, (512, 512, 3))
+
+        c = j.get_codestream()
+        self.assertEqual(c.segment[2].layers, 3)
+
     def test_codeblock_size(self):
         """
         SCENARIO:  Convert TIFF file to JP2 with a specific code block size
@@ -749,6 +808,28 @@ class TestSuite(fixtures.TestCommon):
         self.assertEqual(c.segment[1].xtsiz, 48)
         self.assertEqual(c.segment[1].ytsiz, 48)
 
+    def test_ycbcr_rgb_stripped(self):
+        """
+        SCENARIO:  Convert stripped YCBCR file to JP2.  The TIFF has 2 strips.
+
+        EXPECTED RESULT:  The data matches.  The JP2 file has 4 tiles.
+        """
+        with Tiff2Jp2k(
+            self.ycbcr_stripped, self.temp_jp2_filename, tilesize=(256, 256),
+        ) as j:
+            j.run()
+
+        jp2 = Jp2k(self.temp_jp2_filename)
+        actual = jp2[:]
+        expected = skimage.io.imread(self.ycbcr_stripped, plugin='pil')
+        np.testing.assert_array_equal(actual, expected)
+
+        c = jp2.get_codestream()
+        self.assertEqual(c.segment[1].xsiz, 512)
+        self.assertEqual(c.segment[1].ysiz, 512)
+        self.assertEqual(c.segment[1].xtsiz, 256)
+        self.assertEqual(c.segment[1].ytsiz, 256)
+
     def test_rgb_tiled_bigtiff(self):
         """
         SCENARIO:  Convert RGB BigTIFF file to JP2.  The TIFF is evenly
@@ -872,12 +953,13 @@ class TestSuite(fixtures.TestCommon):
         self.assertEqual(c.segment[1].xtsiz, 512)
         self.assertEqual(c.segment[1].ytsiz, 512)
 
-    def test_rgb_uint16(self):
+    def test_rgb_uint16_tiled(self):
         """
         SCENARIO:  Convert RGB TIFF file to JP2.  The TIFF is evenly
         tiled 2x2 and uint16.
 
-        EXPECTED RESULT:  The data matches.  The JP2 file has 4 tiles.
+        EXPECTED RESULT:  The data matches.  The JP2 file has 4 tiles.  The
+        datatype is uint16.
         """
         with Tiff2Jp2k(
             self.astronaut_u16, self.temp_jp2_filename, tilesize=(32, 32)
@@ -885,6 +967,9 @@ class TestSuite(fixtures.TestCommon):
             j.run()
 
         jp2 = Jp2k(self.temp_jp2_filename)
+
+        self.assertEqual(jp2.box[2].box[0].bits_per_component, 16)
+
         actual = jp2[:]
         expected = skimage.io.imread(self.astronaut_u16)
         np.testing.assert_array_equal(actual, expected)
@@ -895,6 +980,42 @@ class TestSuite(fixtures.TestCommon):
         self.assertEqual(c.segment[1].xtsiz, 32)
         self.assertEqual(c.segment[1].ytsiz, 32)
 
+    def test_rgb_uint16_not_tiled(self):
+        """
+        SCENARIO:  Convert RGB TIFF file to JP2 with one bit jp2k tile.  The
+        datatype is uint16.
+
+        EXPECTED RESULT:  The data matches.  The datatype is uint16.
+        """
+        with Tiff2Jp2k(self.astronaut_u16, self.temp_jp2_filename) as j:
+            j.run()
+
+        jp2 = Jp2k(self.temp_jp2_filename)
+
+        self.assertEqual(jp2.box[2].box[0].bits_per_component, 16)
+
+        actual = jp2[:]
+        expected = skimage.io.imread(self.astronaut_u16)
+        np.testing.assert_array_equal(actual, expected)
+
+    def test_rgb_stripped_uint16_not_tiled(self):
+        """
+        SCENARIO:  Convert stripped uint16 RGB TIFF file to JP2 with one bit
+        jp2k tile.
+
+        EXPECTED RESULT:  The data matches.  The datatype is uint16.
+        """
+        with Tiff2Jp2k(self.astronaut_s_u16, self.temp_jp2_filename) as j:
+            j.run()
+
+        jp2 = Jp2k(self.temp_jp2_filename)
+
+        self.assertEqual(jp2.box[2].box[0].bits_per_component, 16)
+
+        actual = jp2[:]
+        expected = skimage.io.imread(self.astronaut_s_u16)
+        np.testing.assert_array_equal(actual, expected)
+
     def test_commandline_tiff2jp2(self):
         """
         Scenario:  patch sys such that we can run the command line tiff2jp2
@@ -1397,11 +1518,9 @@ class TestSuite(fixtures.TestCommon):
         ColourSpecificationBox.  There is a logging message at the info
         level stating that a color profile was consumed.
         """
-        path = fixtures._path_to('basn6a08.tif')
-
-        with path.open(mode='rb') as f:
-            buffer = f.read()
-            ifd = glymur.lib.tiff.tiff_header(buffer)
+        path = ir.files('tests.data').joinpath('basn6a08.tif')
+        buffer = path.read_bytes()
+        ifd = glymur.lib.tiff.tiff_header(buffer)
         icc_profile = bytes(ifd['ICCProfile'])
 
         with Tiff2Jp2k(
@@ -1430,11 +1549,9 @@ class TestSuite(fixtures.TestCommon):
         Expected Result.  The ICC profile is verified in the
         ColourSpecificationBox.
         """
-        path = fixtures._path_to('basn6a08.tif')
-
-        with path.open(mode='rb') as f:
-            buffer = f.read()
-            ifd = glymur.lib.tiff.tiff_header(buffer)
+        path = ir.files('tests.data').joinpath('basn6a08.tif')
+        buffer = path.read_bytes()
+        ifd = glymur.lib.tiff.tiff_header(buffer)
         icc_profile = bytes(ifd['ICCProfile'])
 
         sys.argv = [
@@ -1457,7 +1574,7 @@ class TestSuite(fixtures.TestCommon):
         profile).  The ICC profile tag will be present in the
         JpgTiffExif->JP2 UUID box.
         """
-        path = fixtures._path_to('basn6a08.tif')
+        path = ir.files('tests.data').joinpath('basn6a08.tif')
 
         sys.argv = ['', str(path), str(self.temp_jp2_filename)]
         command_line.tiff2jp2()
@@ -1482,7 +1599,7 @@ class TestSuite(fixtures.TestCommon):
         ColourSpecificationBox.  The ICC profile tag will be not present in the
         JpgTiffExif->JP2 UUID box.
         """
-        path = fixtures._path_to('basn6a08.tif')
+        path = ir.files('tests.data').joinpath('basn6a08.tif')
 
         sys.argv = [
             '', str(path), str(self.temp_jp2_filename),
@@ -1503,6 +1620,23 @@ class TestSuite(fixtures.TestCommon):
         # the exif UUID box does not have the profile
         self.assertNotIn('ICCProfile', j.box[-1].data)
 
+    def test_num_threads(self):
+        """
+        Scenario:  The --num-threads option is given on the command line.
+
+        Expected Result.  No errors.
+        """
+        path = ir.files('tests.data').joinpath('basn6a08.tif')
+
+        sys.argv = [
+            '', str(path), str(self.temp_jp2_filename), '--num-threads', '2',
+        ]
+        command_line.tiff2jp2()
+
+        Jp2k(self.temp_jp2_filename)
+
+        self.assertTrue(True)
+
     def test_not_a_tiff(self):
         """
         Scenario:  The input "TIFF" is not actually a TIFF.  This used to
@@ -1511,7 +1645,7 @@ class TestSuite(fixtures.TestCommon):
         Expected Result:  no segfault
         """
         with self.assertRaises(RuntimeError):
-            path = fixtures._path_to('simple_rdf.txt')
+            path = ir.files('tests.data').joinpath('simple_rdf.txt')
             with Tiff2Jp2k(path, self.temp_jp2_filename):
                 pass
 
@@ -1529,7 +1663,7 @@ class TestSuite(fixtures.TestCommon):
     def _test_colormap(self, tag):
 
         kwargs = {'tilesize': (32, 32), 'exclude_tags': [tag]}
-        path = fixtures._path_to('issue572.tif')
+        path = ir.files('tests.data').joinpath('issue572.tif')
         with Tiff2Jp2k(path, self.temp_jp2_filename, **kwargs) as p:
             p.run()
 
@@ -1574,7 +1708,7 @@ class TestSuite(fixtures.TestCommon):
         Expected Result:  The UUIDbox has StripOffsets, StripByteCounts, and
         ICCProfile.
         """
-        path = fixtures._path_to('basn6a08.tif')
+        path = ir.files('tests.data').joinpath('basn6a08.tif')
         with Tiff2Jp2k(path, self.temp_jp2_filename, exclude_tags=None) as p:
             p.run()
 
@@ -1595,7 +1729,7 @@ class TestSuite(fixtures.TestCommon):
         """
         with warnings.catch_warnings():
             warnings.simplefilter('ignore')
-            path = fixtures._path_to('albers27.tif')
+            path = ir.files('tests.data').joinpath('albers27.tif')
             with Tiff2Jp2k(path, self.temp_jp2_filename) as j:
                 j.run()
 
@@ -1615,7 +1749,7 @@ class TestSuite(fixtures.TestCommon):
         EXPECTED RESULT:  RuntimeError
         """
         with self.assertRaises(RuntimeError):
-            path = fixtures._path_to('flower-separated-planar-08.tif')
+            path = ir.files('tests.data').joinpath('flower-separated-planar-08.tif')  # noqa : E501
             with Tiff2Jp2k(
                 path, self.temp_jp2_filename, tilesize=(64, 64)
             ) as j:
@@ -1629,7 +1763,7 @@ class TestSuite(fixtures.TestCommon):
         EXPECTED RESULT:  RuntimeError
         """
         with self.assertRaises(RuntimeError):
-            path = fixtures._path_to('albers27-8.tif')
+            path = ir.files('tests.data').joinpath('albers27-8.tif')
             with Tiff2Jp2k(
                 path, self.temp_jp2_filename, tilesize=(256, 256),
             ) as j:
@@ -1642,7 +1776,7 @@ class TestSuite(fixtures.TestCommon):
 
         EXPECTED RESULT:  no errors.
         """
-        path = fixtures._path_to('albers27-8.tif')
+        path = ir.files('tests.data').joinpath('albers27-8.tif')
         with Tiff2Jp2k(path, self.temp_jp2_filename) as j:
             j.run()
 
@@ -1657,3 +1791,113 @@ class TestSuite(fixtures.TestCommon):
             Tiff2Jp2k(
                 self.test_dir_path / 'not_there.tif', self.temp_jp2_filename
             )
+
+    @unittest.skipIf(sys.byteorder == 'big', 'Test simulates big-endian')
+    def test_rgba_interface_big_endian(self):
+        """
+        SCENARIO:  Convert TIFF file to JP2, simulate big endian platform.
+        This test is for code coverage purposes only, since the CI system is
+        currently only little-endian.
+
+        EXPECTED RESULT:  Since a YCbCr image read in thru the RGBA
+        interface on little endian produces RGBA, then throw away the A.
+        Running on little endian but simulating big endian still produces
+        RGBA, but we flip the order and throw away the 4th plane,
+        producing ABG.
+        """
+        with patch('glymur.tiff.sys.byteorder', new='big'):
+            with Tiff2Jp2k(
+                self.astronaut_ycbcr_jpeg_tiled, self.temp_jp2_filename
+            ) as p:
+                p.run()
+
+            j = Jp2k(self.temp_jp2_filename)
+            actual = j[:]
+
+        # Read in the image (as expected on little-endian), flip it
+        # around, prepend an alpha layer, then get rid of the last layer
+        # to simulate what would have happened on big-endian.
+        expected = skimage.io.imread(
+            self.astronaut_ycbcr_jpeg_tiled, plugin='pil'
+        )
+        h, w, _ = expected.shape
+        expected = np.flip(expected, axis=2)
+        A = np.ones((h, w, 1), dtype=np.uint8) * 255
+        expected = np.concatenate((A, expected), axis=2)
+        expected = expected[:, :, :3]
+
+        np.testing.assert_array_equal(actual, expected)
+
+    @unittest.skipIf(sys.byteorder == 'big', 'Test simulates big-endian')
+    def test_rgba_interface_big_endian_tiled(self):
+        """
+        SCENARIO:  Convert TIFF file to JP2, simulate big endian platform.
+        This test is for code coverage purposes only, since the CI system is
+        currently only little-endian.  Use tiling to avoid reading in the
+        entire image at once.
+
+        EXPECTED RESULT:  Since a YCbCr image read in thru the RGBA
+        interface on little endian produces RGBA, then throw away the A.
+        Running on little endian but simulating big endian still produces
+        RGBA, but we flip the order and throw away the 4th plane,
+        producing ABG.
+        """
+        with patch('glymur.tiff.sys.byteorder', new='big'):
+            with Tiff2Jp2k(
+                self.astronaut_ycbcr_jpeg_tiled, self.temp_jp2_filename,
+                tilesize=(32, 32)
+            ) as p:
+                p.run()
+
+            j = Jp2k(self.temp_jp2_filename)
+            actual = j[:]
+
+        # Read in the image (as expected on little-endian), flip it
+        # around, prepend an alpha layer, then get rid of the last layer
+        # to simulate what would have happened on big-endian.
+        expected = skimage.io.imread(
+            self.astronaut_ycbcr_jpeg_tiled, plugin='pil'
+        )
+        h, w, _ = expected.shape
+        expected = np.flip(expected, axis=2)
+        A = np.ones((h, w, 1), dtype=np.uint8) * 255
+        expected = np.concatenate((A, expected), axis=2)
+        expected = expected[:, :, :3]
+
+        np.testing.assert_array_equal(actual, expected)
+
+    @unittest.skipIf(sys.byteorder == 'big', 'Test simulates big-endian')
+    def test_rgba_interface_big_endian_stripped(self):
+        """
+        SCENARIO:  Convert TIFF file to JP2, simulate big endian platform.
+        This test is for code coverage purposes only, since the CI system is
+        currently only little-endian.  Use tiling to avoid reading in the
+        entire image at once.
+
+        EXPECTED RESULT:  Since a YCbCr image read in thru the RGBA
+        interface on little endian produces RGBA, then throw away the A.
+        Running on little endian but simulating big endian still produces
+        RGBA, but we flip the order and throw away the 4th plane,
+        producing ABG.
+        """
+        with patch('glymur.tiff.sys.byteorder', new='big'):
+            with Tiff2Jp2k(
+                self.ycbcr_stripped, self.temp_jp2_filename,
+                tilesize=(32, 32)
+            ) as p:
+                p.run()
+
+            j = Jp2k(self.temp_jp2_filename)
+            actual = j[:]
+
+        # Read in the image (as expected on little-endian), flip it
+        # around, prepend an alpha layer, then get rid of the last layer
+        # to simulate what would have happened on big-endian.
+        expected = skimage.io.imread(self.ycbcr_stripped, plugin='pil')
+        h, w, _ = expected.shape
+        expected = np.flip(expected, axis=2)
+        A = np.ones((h, w, 1), dtype=np.uint8) * 255
+        expected = np.concatenate((A, expected), axis=2)
+        expected = expected[:, :, :3]
+
+        np.testing.assert_array_equal(actual, expected)
